from datetime import datetime, timedelta
from dbsynce.lib.core import get_admin_url
from dbsynce.models import DocumentFile
from tickets.models import Ticket, TicketList

#Внутренние роли для обеспечения функционирования Сервиса
def create_ticket_role_activation_metaservice_supervisor(user):
    """
    Создание тикета на получение роли Модератора сервиса.
    
    Список: 2102, METASERVICE-ADMIN: Присвоение роли Модератора Сервиса (ACCESS_REQUEST)
    Тип: 3, ACCESS_REQUEST
    """
    return Ticket.objects.create(
        title=f"Запрос прав на становление Модератором сервиса'",
        ticket_list=TicketList.objects.get(pk=2102),  # Список кому назначается: METASERVICE-ADMIN
        ticket_type=3,  # ACCESS_REQUEST
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,

        # FIXME: Возможно нужно автоматически создавать содержимое поля json
        note=f"""
            Пользователь {user} #{user.pk} отправил заявку на роль Модератора Сервиса.\n
            \n
            После принятия изменений назначается роль и пользователь получает право быть Модератором Сервиса.
        """
    )

def create_ticket_role_activation_metaservice_support(user):
    """
    Создание тикета на получение роли Поддержки сервиса.
    
    Список: 2202, METASERVICE-SUPERVISOR: Присвоение роли Поддержки Сервиса (ACCESS_REQUEST)
    Тип: 3, ACCESS_REQUEST
    """
    return Ticket.objects.create(
        title=f"Запрос прав на становление Поддержкой сервиса'",
        ticket_list=TicketList.objects.get(pk=2202),  # Список кому назначается: METASERVICE-SUPERVISOR
        ticket_type=3,  # ACCESS_REQUEST
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,

        # FIXME: Возможно нужно автоматически создавать содержимое поля json
        note=f"""
            Пользователь {user} #{user.pk} отправил заявку на роль Поддержки Сервиса.\n
            \n
            После принятия изменений назначается роль и пользователь получает право быть Сотрудником поддержки.
        """
    )
    
def create_ticket_role_activation_metaservice_techsupport(user):
    """
    Создание тикета на получение роли Поддержки сервиса.
    
    Список: 2202, METASERVICE-SUPERVISOR: Присвоение роли Техподдержки Сервиса (ACCESS_REQUEST)
    Тип: 3, ACCESS_REQUEST
    """
    return Ticket.objects.create(
        title=f"Запрос прав на становление Техподдержкой сервиса'",
        ticket_list=TicketList.objects.get(pk=2202),  # Список кому назначается: METASERVICE-SUPERVISOR
        ticket_type=3,  # ACCESS_REQUEST
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,

        # FIXME: Возможно нужно автоматически создавать содержимое поля json
        note=f"""
            Пользователь {user} #{user.pk} отправил заявку на роль Техподдержки Сервиса.\n
            \n
            После принятия изменений назначается роль и пользователь получает право быть Сотрудником техподдержки.
        """
    )

#Назначение прав Партнерам Сервиса
def create_ticket_role_activation_partner_admin(user, сompany):
    """
    Создание тикета на получение роли администратора партнера сервиса.
    
    Список: 2102, METASERVICE-ADMIN: Присвоение роли Администратора Партнера (ACCESS_REQUEST)
    Тип: 3, ACCESS_REQUEST
    """
    return Ticket.objects.create(
        title=f"Запрос прав на получение роли Партнера на основе создания партнерского кабинета '{сompany.legal_name}'",
        ticket_list=TicketList.objects.get(pk=2102),  # Список кому назначается: METASERVICE-ADMIN
        ticket_type=3,  # ACCESS_REQUEST
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,

        # FIXME: Возможно нужно автоматически создавать содержимое поля json
        note=f"""
            Пользователь {user} #{user.pk} отправил заявку на роль администратора партнера сервиса.\n
            \n
            <a href="{get_admin_url(сompany)}">Полная информация о компании-партнере</a>\n
            \n
            После принятия изменений назначается роль и получает право на совершение юридически значимых действий от лица Партнера в соответствии с функционалом Сервиса.
        """
    )
    
def create_ticket_role_activation_partner_supervisor(user, сompany):
    """
    Создание тикета на получение роли Модератора Партнера сервиса.
    
    Список: 3102, PARTNER-ADMIN: Присвоение роли Модератора партнера(ACCESS_REQUEST)
    Тип: 3, ACCESS_REQUEST
    """
    return Ticket.objects.create(
        title=f"Запрос прав на становление Модератором партнера '{сompany.legal_name}'",
        ticket_list=TicketList.objects.get(pk=3102),  # Список кому назначается: METASERVICE-ADMIN
        ticket_type=3,  # ACCESS_REQUEST
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,

        # FIXME: Возможно нужно автоматически создавать содержимое поля json
        note=f"""
            Пользователь {user} #{user.pk} отправил заявку на роль модератора партнера сервиса.\n
            \n
            <a href="{get_admin_url(сompany)}">Полная информация о компании-партнере</a>\n
            \n
            После принятия изменений назначается роль и получает право модерировать процессы Партнера.
        """
    )
    
def create_ticket_role_activation_partner_support(user, сompany):
    """
    Создание тикета на получение роли поддержки партнера сервиса.
    
    Список: 3202, PARTNER-SUPERVISOR: Присвоение роли Поддержки партнера(ACCESS_REQUEST)
    Тип: 3, ACCESS_REQUEST
    """
    return Ticket.objects.create(
        title=f"Запрос прав на становление Поддержки партнера '{сompany.legal_name}'",
        ticket_list=TicketList.objects.get(pk=3202),  # Список кому назначается: PARTNER-SUPERVISOR
        ticket_type=3,  # ACCESS_REQUEST
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,

        # FIXME: Возможно нужно автоматически создавать содержимое поля json
        note=f"""
            Пользователь {user} #{user.pk} отправил заявку на роль поддержки партнера сервиса.\n
            \n
            <a href="{get_admin_url(сompany)}">Полная информация о компании-партнере</a>\n
            \n
            После принятия изменений назначается роль сотрудника поддержки партнера.
        """
    )
    
def create_ticket_role_activation_partner_techsupport(user, сompany):
    """
    Создание тикета на получение роли техподдержки партнера сервиса.
    
    Список: 3202, PARTNER-SUPERVISOR: Присвоение роли ТехПоддержки партнера(ACCESS_REQUEST)
    Тип: 3, ACCESS_REQUEST
    """
    return Ticket.objects.create(
        title=f"Запрос прав на становление ТехПоддержки партнера '{сompany.legal_name}'",
        ticket_list=TicketList.objects.get(pk=3202),  # Список кому назначается: PARTNER-SUPERVISOR
        ticket_type=3,  # ACCESS_REQUEST
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,

        # FIXME: Возможно нужно автоматически создавать содержимое поля json
        note=f"""
            Пользователь {user} #{user.pk} отправил заявку на роль техподдержки партнера сервиса.\n
            \n
            <a href="{get_admin_url(сompany)}">Полная информация о компании-партнере</a>\n
            \n
            После принятия изменений назначается роль сотрудника техподдержки партнера.
        """
    )

#Назначение роли исполнителя в Сервисе
def create_ticket_role_activation_provider(user, provider):
    """
    Создание тикета на получение роли Исполнителя сервиса.
    
    Список: 2202, METASERVICE-SUPERVISOR: Присвоение роли (ACCESS_REQUEST)
    Тип: 3, ACCESS_REQUEST
    """
    return Ticket.objects.create(
        title=f"Запрос прав на становление Исполнителем на основе первой заявки от '{provider.company}'",
        ticket_list=TicketList.objects.get(pk=2202),  # Список кому назначается: METASERVICE-ADMIN
        ticket_type=3,  # ACCESS_REQUEST
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,

        # FIXME: Возможно нужно автоматически создавать содержимое поля json
        note=f"""
            Пользователь {user} #{user.pk} отправил заявку на роль Исполнителя партнера '{provider.company}'.\n
            \n
            <a href="{get_admin_url(provider.сompany)}">Полная информация о компании-партнере</a>\n
            \n
            После принятия изменений назначается роль и пользователь получает право быть Исполнителем в Сервисе.
        """
    )

#Блок тикетов на создание запросов на верификацию документов разного уровня

def create_ticket_metaservice_docs_verification(user, doc):
    """
    Создание тикета на проверку документов сервиса.
    
    Список: 2205, METASERVICE-SUPERVISOR: Проверка документов (ST_REQUEST)
    Тип: 1, ST_REQUEST
    """
    doc_name = doc.get_doc_type_display()
    doc_files = DocumentFile.objects.filter(document=doc)

    note = f"Пользователь {user} #{user.pk} добавил новые файлы документа <a href='{get_admin_url(doc)}'>{doc_name}</a> сервиса, требующие проверки:<ul>"
    for file in doc_files:
        note += f"<li><a href='{file.file.url}' target='_blank'>{file}</a></li>"
    note += "</ul><br>Успешная проверка обозначается выставелнием статуса DONE."

    return Ticket.objects.create(
        title=f"Проверка документа '{doc_name}' пользователя",
        ticket_list=TicketList.objects.get(pk=2205),
        ticket_type=1,
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,
        note=note
    )

def create_ticket_company_docs_verification(user, company, doc):
    """
    Создание тикета на проверку документов партнера.
    
    Список: 3205, PARTNER_SUPERVISOR: Проверка документов (ST_REQUEST)
    Тип: 1, ST_REQUEST
    """
    doc_name = doc.get_doc_type_display()
    doc_files = DocumentFile.objects.filter(document=doc)

    note = f"Пользователь {user} #{user.pk} добавил новые файлы документа <a href='{get_admin_url(doc)}'>{doc_name}</a> партнера <a href='{get_admin_url(company)}'>{company.legal_name}</a> требующие проверки:<ul>"
    for file in doc_files:
        note += f"<li><a href='{file.file.url}' target='_blank'>{file}</a></li>"
    note += "</ul><br>Успешная проверка обозначается выставелнием статуса DONE."

    return Ticket.objects.create(
        title=f"Проверка документа '{doc_name}' партнера '{company.legal_name}'",
        ticket_list=TicketList.objects.get(pk=3205),
        ticket_type=1,
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,
        note=note
    )
    
#Блок тикетов на создание запросов на верификацию проверок для допусков (экзаменов) разного уровня

def create_ticket_metaservice_exam_verification(user, doc):
    """
    Создание тикета на проверку экзаменов (допусков).
    
    Список: 2206, METASERVICE-SUPERVISOR: Проверка допусков (ST_REQUEST)
    Тип: 1, ST_REQUEST
    """
    doc_name = doc.get_doc_type_display()
    doc_files = DocumentFile.objects.filter(document=doc)

    note = f"Пользователю {user} #{user.pk} необходима проверка докусков на данном уровне требования. <br>Успешная проверка обозначается выставелнием статуса DONE."

    return Ticket.objects.create(
        title=f"Проверка допусков '{doc_name}' пользователя",
        ticket_list=TicketList.objects.get(pk=2206),
        ticket_type=1,
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,
        note=note
    )

def create_ticket_company_docs_verification(user, company, doc):
    """
    Создание тикета на проверку документов партнера.
    
    Список: 3206, PARTNER_SUPERVISOR: Проверка документов (ST_REQUEST)
    Тип: 1, ST_REQUEST
    """
    doc_name = doc.get_doc_type_display()
    doc_files = DocumentFile.objects.filter(document=doc)

    note = f"Пользователь {user} #{user.pk} необходима проверка докусков на данном уровне требования. <br>Успешная проверка обозначается выставелнием статуса DONE."


    return Ticket.objects.create(
        title=f"Проверка допусков партнером '{company.legal_name}'",
        ticket_list=TicketList.objects.get(pk=3206),
        ticket_type=1,
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,
        note=note
    )

#Блок тикетов на активацию юридических неодушевленных единиц

def create_ticket_service_category_activation(user, service_category):
    """
    Создание тикета на активацию Категории Сервиса.
    
    Список: 21031, METASERVICE-ADMIN: Активация Типов услуг (категорий сервисов) (ST_REQUEST)
    Тип: 1, ST_REQUEST
    """
    # FIXME: Возможно нужно автоматически создавать содержимое поля json
    
    #TODO вместе с созданием тикета должно отправляться уведомление в группу, но может быть это будет на уровне обработчиков
    
    return Ticket.objects.create(
        title=f"Активация новой категории сервиса '{service_category.id_metaservice}' - '{service_category.caption}'",
        ticket_list=TicketList.objects.get(pk=21031),
        ticket_type=1,
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,

        note=f"""
            Пользователь {user} отправил заявку на добавление и активацию категории сервиса:\n
            - Название: {service_category.caption}\n
            - Договор: {service_category.link_agreement}\n
            - Комиссия Сервиса: {service_category.metaservice_comission}\n
            - Тарификация: {service_category.price_type}\n
            - Описание: {service_category.description}\n
            \n
            Проверьте всю информацию и для активации категории сервиса измените статус заявки на ACCEPTED.
            Это будет означать, что Сервис сможет обрабатывать заказы данной категории.
        """
    )
    
    
def create_ticket_service_activation(user, service):
    """
    Создание тикета на активацию Услуги (согласование тарифов).
    
    Список: 21032, METASERVICE-SUPERVISOR: Активация услуг (ST_REQUEST)
    Тип: 1, ST_REQUEST
    """
    # FIXME: Возможно нужно автоматически создавать содержимое поля json
    
    #TODO вместе с созданием тикета должно отправляться уведомление в группу, но может быть это будет на уровне обработчиков
    
    return Ticket.objects.create(
        title=f"Активация новой услуги '{service.id_metaservice}' - '{service.caption}' (согласование тарифов)",
        ticket_list=TicketList.objects.get(pk=21032),
        ticket_type=1,
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,
        #TODO надо сделать более информативный вывод
        note=f"""
            Пользователь {user} отправил заявку на добавление и активацию услуги сервиса:\n
            - Категория: {service.service_category_id}\n
            - Ресурс: {service.resource}\n
            - Комиссия партнера: {service.company_comission}\n
            - Тарификация: {service.price_type}\n
            - Стоимость км: {service.price_km}\n
            - Стоимость минуты: {service.price_min}\n            
            - Стоимость за количество: {service.price_amount}\n
            - Описание: {service_category.description}\n
            \n
            Проверьте всю информацию и для активации категории сервиса измените статус заявки на ACCEPTED.
            Это будет означать, что Партнер сможет оказывать услуги на указанных условиях.
        """
    )

def create_ticket_company_activation(user, сompany):
    """
    Создание тикета на активацию партнера.
    
    Список: 21033, METASERVICE-ADMIN: Активация партнеров (ST_REQUEST)
    Тип: 1, ST_REQUEST
    """
    # FIXME: Возможно нужно автоматически создавать содержимое поля json
    return Ticket.objects.create(
        title=f"Активация нового Партнера '{сompany.legal_name}'",
        ticket_list=TicketList.objects.get(pk=21033),
        ticket_type=4,
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,

        note=f"""
            Пользователь {user} #{user.pk} отправил заявку на становление партнером сервиса:\n
            - Имя: {сompany.legal_name}\n
            - ИНН: {сompany.inn}\n
            - Юридический адрес: {сompany.address}\n
            <a href="{get_admin_url(сompany)}">Полная информация</a>\n
            \n
            Проверьте всю информацию и для активации партнера измените статус заявки на ACCEPTED.
            Это будет означать, что договорные отношения между сервисом и партнером, вступают в силу.
        """
    )

def create_ticket_provider_activation_by_metaservice_supervisor(user, provider):
    """
    Создание тикета на активацию Исполнителя на оказание конкретной услуги.
    
    Список: 22034, METASERVICE-SUPERVISOR: Активация услуг (ST_REQUEST)
    Тип: 1, ST_REQUEST
    """
    # FIXME: Возможно нужно автоматически создавать содержимое поля json
    
    #TODO вместе с созданием тикета должно отправляться уведомление в группу, но может быть это будет на уровне обработчиков
    
    return Ticket.objects.create(
        title=f"Активация услуги '{provider.service}' для исполнителя '{user}'",
        ticket_list=TicketList.objects.get(pk=22034),
        ticket_type=1,
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,
        #TODO надо сделать более информативный вывод
        note=f"""
            Пользователь {user} отправил заявку на оказание Услуги {provider.service}.\n

            Проверьте всю информацию о необходимых разрешениях и проверках и для активации возможности оказывать такие услуги измените статус заявки на ACCEPTED.
            Это будет означать, что Исполнитель сможет обрабатывать такие заказы.
        """
    )
    
def create_ticket_resource_activation_by_metaservice_supervisor(user, resource):
    """
    Создание тикета на активацию Ресурса.
    
    Список: 22035, METASERVICE-SUPERVISOR: Активация Ресурса (ST_REQUEST)
    Тип: 1, ST_REQUEST
    """
    # FIXME: Возможно нужно автоматически создавать содержимое поля json
    
    #TODO вместе с созданием тикета должно отправляться уведомление в группу, но может быть это будет на уровне обработчиков
    
    return Ticket.objects.create(
        title=f"Активация Ресурса '{resource.resource_type}' для пользователя '{user}'",
        ticket_list=TicketList.objects.get(pk=22035),
        ticket_type=1,
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,
        #TODO надо сделать более информативный вывод
        note=f"""
            Пользователь {user} отправил заявку на добавление в систему ресурса вида {resource.resource_type}.\n

            Проверьте всю информацию о необходимых разрешениях и проверках и для активации возможности оказывать такие услуги измените статус заявки на ACCEPTED.
            Это будет означать, что Исполнитель сможет обрабатывать такие заказы.
        """
    )
    
def create_ticket_provider_activation_by_partner_admin(user, provider):
    """
    Создание тикета на активацию Исполнителя на оказание конкретной услуги.
    
    Список: 31034, PARTNER-ADMIN: Активация Исполнителя на услугу (ST_REQUEST)
    Тип: 1, ST_REQUEST
    """
    # FIXME: Возможно нужно автоматически создавать содержимое поля json
    
    #TODO вместе с созданием тикета должно отправляться уведомление в группу, но может быть это будет на уровне обработчиков
    
    return Ticket.objects.create(
        title=f"Активация услуги '{provider.service}' для исполнителя '{user}'",
        ticket_list=TicketList.objects.get(pk=32034),
        ticket_type=1,
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,
        #TODO надо сделать более информативный вывод
        note=f"""
            Пользователь {user} отправил заявку на оказание Услуги {provider.service}.\n

            Проверьте всю информацию о необходимых разрешениях и проверках и для активации возможности оказывать такие услуги измените статус заявки на ACCEPTED.
            Это будет означать, что Исполнитель сможет обрабатывать такие заказы.
        """
    )
    
def create_ticket_resource_activation_by_partner_admin(user, resource):
    """
    Создание тикета на активацию Ресурса.
    
    Список: 31035, PARTNER-ADMIN: Активация Ресурса (ST_REQUEST)
    Тип: 1, ST_REQUEST
    """
    # FIXME: Возможно нужно автоматически создавать содержимое поля json
    
    #TODO вместе с созданием тикета должно отправляться уведомление в группу, но может быть это будет на уровне обработчиков
    
    return Ticket.objects.create(
        title=f"Активация Ресурса '{resource.resource_type}' для пользователя '{user}'",
        ticket_list=TicketList.objects.get(pk=31035),
        ticket_type=1,
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,
        #TODO надо сделать более информативный вывод
        note=f"""
            Пользователь {user} отправил заявку на добавление в систему ресурса вида {resource.resource_type}.\n

            Проверьте всю информацию о необходимых разрешениях и проверках и для активации возможности оказывать такие услуги измените статус заявки на ACCEPTED.
            Это будет означать, что Исполнитель сможет обрабатывать такие заказы.
        """
    )
    
#Набор тикетов для установления договорных взаимоотношений
#FIXME - он пока неправильный, над ним надо подумать, как и над его целесообразностью!

def create_ticket_contract_with_metaservice(user, сompany):
    """
    Создание тикета на установление договорных взаимоотношений.
    
    Список: 2101, METASERVICE-ADMIN: Установление договорных взаимоотношений (NEG_REQUEST)
    Тип: 4, NEG_REQUEST
    """
    # FIXME: Возможно нужно автоматически создавать содержимое поля json
    return Ticket.objects.create(
        title=f"Установление договорных взаимоотношений с Партнером '{сompany.legal_name}'",
        ticket_list=TicketList.objects.get(pk=2101),
        ticket_type=4,
        due_date=datetime.now().date() + timedelta(days=30),
        created_by=user,

        note=f"""
            Пользователь {user} #{user.pk} отправил заявку на установление партнером сервиса:\n
            - Имя: {сompany.legal_name}\n
            - ИНН: {сompany.inn}\n
            - Юридический адрес: {сompany.address}\n
            <a href="{get_admin_url(сompany)}">Полная информация</a>\n
            \n
            Проверьте всю информацию и для установления договорных взаимоотношений с партнером измените статус заявки на ACCEPTED.
            Это будет означать, что договорные отношения между сервисом и партнером, вступают в силу.
        """
    )

